<?php
namespace Modules\EST\Services;

use \Geral\Entities\Produto;
use \Estoque\Entities\Itlanctoest;
use \Teknisa\PebLibs\Services\ValidacoesEstoqueService;
use \Doctrine\ORM\EntityManager;
use \Modules\EST\Factory\AjusteFactory;

class AjusteService {

    protected $AjusteFactory;
    protected $nrorg;
    protected $filialLogada;
    protected $operador;
    protected $newCode;
    protected $entityManager;
    protected $ValidacoesEstoqueService;

    public function __construct (AjusteFactory $AjusteFactory, $environmment, $newCode, $entityManager,$ValidacoesEstoqueService) {
        $this->AjusteFactory = $AjusteFactory;
        $this->ValidacoesEstoqueService = $ValidacoesEstoqueService;
        $this->nrorg = $environmment->getNrorg();
        //$this->nrorg = 1;
        $this->filialLogada = $environmment->getCdFilial();
        $this->operador = $environmment->getCdOperador();
        $this->entityManager = $entityManager;
        $this->newCode = $newCode;
    }
    
    public function containsProducts ($productsArray) {
        if (count($productsArray) <= 0) {
            throw new \Exception("Erro. Um dos ajustes está sem produto", 1);
        }
    }

    public function generatedValidSequential () {
        $filial = $this->filialLogada;
        return $this->newCode->getCodeByTableName('LANCTOEST'.$filial, $this->nrorg);
    }

    public function setDefaultColumnsForAdjustment ($adjustment) {
        $adjustment['CDFILIMOVI'] = $adjustment['CDFILIAL']? $adjustment['CDFILIAL'] : $this->filialLogada;
        $adjustment['CDFILIAL'] =  $this->filialLogada;
        $adjustment['IDTPLANCTO'] = '1';
        $adjustment['IDALTCUSMED'] = 'N';
        $adjustment['IDSTATUSCTBEST'] = 'P';
        $adjustment['CDOPERADOR'] = $this->operador;
        $adjustment['NRORG'] = $this->nrorg;
        $adjustment['SITUACAO'] = 'E';

        return $adjustment;
    }

    public function setValidSequential ($nrlancestq, $products) {
        foreach ($products as &$product) {
            $product['NRLANCESTQ'] = $nrlancestq;
        }

        return $products;
    }

    public function checkRelationship ($nrlancestq, $products) {
        foreach ($products as &$product) {
            if ($product['NRLANCESTQ'] != $nrlancestq) {
                throw new \Exception("O produto ".$product['NMPROUTO']." do ajuste de sequencial "."$nrlancestq"." não tem o mesmo sequencial do seu ajuste.", 1);                
            }
        }
    }

    public function setDefaultColumnsForProducts ($products, $date, $filial) {
        $this->setProduto($products);
        foreach ($products as $key => &$product) {
            //Rotina para revalidar e atualizar vinculo com produto de estoque.
            if(!isset($product['PRODUTO'])){
                continue;
                //pula produtos problematicos do loop para evitar erros.
            }
            $product['PRODUTO'] = $product['CDPRODUTO'];
            $productInfo = $this->getProductInfo($product['PRODUTO']);
            if(empty($productInfo)){
                throw new \Exception('Produto não encontrado');
            }
            $prodEsto = !isset($product['CDPRODESTO'])? $productInfo->getCdProdesto() : $product['CDPRODESTO'];
            
            $product['NRSEQUITEM'] = str_pad($key+1, 4, '0', STR_PAD_LEFT);
            $product['IDTIPOMOVI'] = '1';
            $product['CDFILIMOVI'] = $filial? $filial : $this->filialLogada;
            $product['CDFILIAL'] = $this->filialLogada;
            $product['VRLANCTOBRUT'] = 0;
            $product['IDEVOLESTOQ'] = 'N';
            $product['CDPRODMOVI'] = $prodEsto;
            $product['QTLANCTOEST'] = $product['QTTOTLANCTO'] * $productInfo->getVrfatoconv();
            $product['DTLANCMOVI'] = $date;
            $product['CDOPERADOR'] = $this->operador;
            $product['NRORG'] = $this->nrorg;

            if (empty($product['NRLOTEESTQ'])) {
                $product['NRLOTEESTQ'] = " ";
            }
            if (empty($product['NRSUBLOTE'])) {
                $product['NRSUBLOTE'] = " ";
            }
            if (empty($product['CDALMOXARIFE'])) {
                $product['CDALMOXARIFE'] = " ";
            }
            if (empty($product['CDLOCALESTOQ'])) {
                $product['CDLOCALESTOQ'] = " ";
            }
        }
        
        $products = array_filter($products, function($product){
        	return isset($product['PRODUTO']);
        });
        
        return $products;
    }
    
    public function setProduto(&$products){
        $fields = ['CDPRODINTE','CDARVPROD','CDBARPRODUTO','CDPRODUTO'];
        foreach($fields as &$fieldname){
            if(isset($products[0][$fieldname])){
                foreach($products as &$product){
                    $product['PRODUTO'] = $product[$fieldname];
                }
            }
        }
    }
    
    public function getProductInfo($product){
        
        $fields = ['cdprodinte','cdarvprod','cdbarproduto','cdproduto']; //na entidade tem q ser minusculo
        
        foreach($fields as $field){
            $productInfo = $this->entityManager->getRepository(Produto::class)
                ->findOneBy(
                    [
                        $field => $product,
                        'nrorg' => $this->nrorg
                    ]
                );
            if(!is_null($productInfo)){
                return $productInfo;
            }
        }
    }

    public function saveAdjustment ($adjustment) {
        if(!isset($adjustment['CDALMOXARIFE']))
            $adjustment['CDALMOXARIFE'] = ' ';
        if(!isset($adjustment['CDLOCALESTOQ']))
            $adjustment['CDLOCALESTOQ'] = ' ';
        
        $parameters = array(
            "NRLANCESTQ" => $adjustment['NRLANCESTQ'],
            "DTLANCESTQ" => $adjustment['DTLANCESTQ'],
            "DSLANCESTQ" => $adjustment['DSLANCESTQ'],
            "CDFILIAL" => $adjustment['CDFILIAL'],
            "CDFILIMOVI" => $adjustment['CDFILIMOVI'],
            "IDTPLANCTO" => $adjustment['IDTPLANCTO'],
            "IDALTCUSMED" => $adjustment['IDALTCUSMED'],
            "IDSTATUSCTBEST" => $adjustment['IDSTATUSCTBEST'],
            "CDALMOXARIFE" => $adjustment['CDALMOXARIFE'],
            "CDLOCALESTOQ" => $adjustment['CDLOCALESTOQ'],
            "CDOPERADOR"=> $adjustment['CDOPERADOR'],
            "NRORG" => $adjustment['NRORG']
        );
        $this->AjusteFactory->insertLanctoestoq($parameters);
    }

    public function saveProducts ($product) {
        if(!isset($product['IDAJUSTEIMPORT']))
            $product['IDAJUSTEIMPORT'] = 'N';
        $product['IDAJUSTEIMPORT'] = $product['IDAJUSTEIMPORT'] == 'OK'? 'S' : 'N';
        
        $parameters = array(
            "NRLANCESTQ" => $product['NRLANCESTQ'],
            "NRSEQUITEM" => $product['NRSEQUITEM'],
            "CDPRODUTO" => $product['PRODUTO'],
            "NRLOTEESTQ" => $product['NRLOTEESTQ'],
            "NRSUBLOTE" => $product['NRSUBLOTE'],
            "CDALMOXARIFE" => $product['CDALMOXARIFE'],
            "CDLOCALESTOQ" => $product['CDLOCALESTOQ'],
            "CDFILIAL" => $product['CDFILIAL'],
            "CDFILIMOVI" => $product['CDFILIMOVI'],
            "QTLANCTOEST" => $product['QTLANCTOEST'],
            "IDEVOLESTOQ" => $product['IDEVOLESTOQ'],
            "IDTIPOMOVI" => is_null($product['IDTIPOMOVI'])? '1' : $product['IDTIPOMOVI'],
            "VRLANCTOBRUT" => $product['VRLANCTOBRUT'],
            "CDPRODMOVI" => $product['CDPRODMOVI'],
            "DTLANCMOVI" => $product['DTLANCMOVI'],
            "QTTOTLANCTO" => $product['QTTOTLANCTO'],
            "CDOPERADOR" => $product['CDOPERADOR'],
            "IDAJUSTEIMPORT" => $product['IDAJUSTEIMPORT'],
            "NRORG" => $product['NRORG']
        );
        $this->AjusteFactory->insertItlanctoest($parameters);
    }
    
    public function changeIsNew ($adjustment) {
        $adjustment['__is_new'] = false;
        
        foreach ($adjustment['PRODUCTS'] as &$product) {
            $product['__is_new'] = false;
        }
        return $adjustment;
    }
    
    public function getParamAppAtuEstq($filial){
        try{
            $param = $this->AjusteFactory->getParamAppAtuEstq($filial)[0]['IDAPPATUESTQ'];
            return $param == 'S'? true : false;
        }catch (\Exception $e){
            return false;
        }
    }
    
    public function setEntityManager($entityManager) {
        $this->entityManager = $entityManager;
    }

    public function getEntityManager() {
        return $this->entityManager;
    }
}